
#ifndef _cs_h

#define _cs_h

/******************************************************************************/
/* Include files: ------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "chunk.h"

#ifdef MALLOC_CHECK
#include "zmalloc.h"
#endif

#undef WM_CONTINUOUS
#define  WM_DISCRETE

/******************************************************************************/

typedef enum {HAND, MAX_RESOURCES} Resource;

#define MIN(A,B)       ((A)<(B)?(A):(B))
#define MAX(A,B)       ((A)>(B)?(A):(B))

#define FUZZY_GOALS

#define FZAND(A,B)             MIN(A,B)
#define FZOR(A,B)              MAX(A,B)
#define FZNOT(A)               (1 - A)
#define LOGICAL_AND(A,B)       ((A) && (B))
#define LOGICAL_OR(A,B)        ((A) || (B))

#ifdef FUZZY_GOALS

#define GoalType      double
#define GOAL_TRUE     1.0
#define GOAL_FALSE    0.0

#else

#define GoalType      Boolean
#define GOAL_TRUE     TRUE
#define GOAL_FALSE    FALSE

#endif

/******************************************************************************/
/* Global Constants: ---------------------------------------------------------*/

#define MaxCycles      250000

/******************************************************************************/
/* Global Type Definitions: --------------------------------------------------*/

typedef enum {SCHEMA, OBJECT, EFFECTOR, MAX_COMPONENTS} ComponentType;
typedef enum {SIGMOID, TANH, HALFX, M_R, HOUGHTON, NORMALISED} MathsType;
typedef enum {RUN, SELECTION, BIND, WARNING, WORLD, MONITOR, MAX_MESSAGE_TYPES} MessageType;
typedef enum {UNKNOWN_LEVEL, HIGH, MEDIUM, LOW} SchemaLevel;
typedef enum {CURRENT, SUM, SUMSQ, AVERAGE, STDDEV} CountType;
typedef enum {SOURCE, TARGET, THEME, IMPLEMENT, EMPTY, MAX_ROLES} ObjectRole;

/******************************************************************************/
/* Global Functions: **********************************************************/

extern Boolean allocCheck(void *p, char *d);
extern Boolean nullCheck(void *p, char *d);
extern void exitMessage(char *d);
extern Boolean stoi(char *string, int *intptr);
extern void printSpace(int i, FILE *fp);
extern FILE *update_trial_number(int *i);

/* Object features: ***********************************************************/

#define IS_NUMBER_ONE             1
#define IS_LETTER_A               1
#define IS_NUMBER_TWO             2
#define IS_LETTER_B               2
#define IS_NUMBER_THREE           4
#define IS_LETTER_C               4
#define IS_NUMBER_FOUR            8
#define IS_LETTER_D               8
#define IS_LETTER_E              16
#define IS_LETTER_F              32
#define IS_LETTER_G              64
#define IS_LETTER_H             128
#define IS_LETTER_I             256
#define IS_LETTER_J             512

#define IS_FORM_TRIANGLE       1024
#define IS_FORM_STAR           2048
#define IS_FORM_CROSS          4096
#define IS_FORM_CIRCLE         8192

#define IS_COLOUR_RED         16384
#define IS_COLOUR_GREEN       32768
#define IS_COLOUR_YELLOW      65536
#define IS_COLOUR_WHITE       65536
#define IS_COLOUR_BLUE       131072

#define FEEDBACK_CORRECT     262144
#define FEEDBACK_WRONG       524288

#define IS_PEG               262144
#define IS_BALL              524288

#define IS_DISK              262144

#define IS_APPARATUS        1048576
#define IS_GOAL_STATE       2097152

/**

4194304
8388608
16777216
33554432
67108864
134217728
268435456
536870912
1073741824

**/

/* State info: ****************************************************************/

#define STATE_OPEN                1
#define STATE_CLOSED              2
#define STATE_MOIST               4
#define STATE_ADDRESSED           8
#define STATE_FOLDED             16
#define STATE_STAMPED            32
#define STATE_CUT                64
#define STATE_LIT               128
#define STATE_USED              256
#define STATE_BOUNCED           512

/******************************************************************************/
/* Class Definitions: *********************************************************/

typedef struct task_data {
    TaskId          id;
    void (*install_schemas)();
    void (*install_objects)();
    void (*install_effectors)();
    void (*write_object_attributes)();
    void (*write_effector_attributes)();
    void (*analyse)();
} TaskData;

typedef struct wme {
    Chunk     *content;
    double     activation[MaxCycles];
    Boolean    highlighted;
    int        timestamp;
    double     input;
} Wme;

typedef struct wme_list {
    Wme             *head;
    struct wme_list *tail;
} WmeList;

typedef struct schema_name_list {
    char                    *name;
    struct schema_name_list *next;
} SchemaNameList;

typedef struct contention_scheduler {
    TaskData                 task;
    int                      cycle;
    char                    *logfile;
    struct schema_list      *schemas;
    struct object_list      *objects;
    struct effector_list    *effectors;
    struct object           *current_object[MAX_ROLES];
    struct object           *fixated_object;
    struct schema_list      *update_goal_queue;
    struct schema_list      *selection_queue;
    struct schema_list      *deselection_queue;
    struct schema_list      *execution_queue;
    struct actions          *actions;
    void                    *cl_actions;
    void                    *cl_analysis;
    void                    *cl_schemas;
    void                    *cl_objects[MAX_ROLES];
    void                    *cl_effectors;
    char                    *temporary_schema;
    struct wme_list         *wmes;
    struct schema_name_list *strategy;
} ContentionScheduler;

typedef struct goal {
    char *name;
    Boolean explicitly_achieved;
    GoalType (*precondition)();
    GoalType (*postcondition)();
} Goal;

typedef struct goal_list {
    Goal *head;
    struct goal_list *tail;
} GoalList;

typedef struct object {
    char *name;
    double activation[MAX_ROLES][MaxCycles];
    double (*objectTriggers)(ContentionScheduler *cs, void *o, ObjectRole role);
    Boolean inhibit[MAX_ROLES];
    /* Physical aspects of the object: */
    unsigned int features;
    unsigned int state;
    double position[2];
    struct effector *holder;
    struct object_list *itemsContained,  *itemsCarried, *itemsAttached;
    struct object *container, *carrier, *base;
    /* For purposes of the graph: */
    int time0;
    Boolean highlighted;
} Object;

typedef struct object_list {
    Object *head;
    struct object_list *tail;
} ObjectList;

typedef struct effector {
    char *name;
    double activation[MaxCycles];
    ObjectList *itemsHeld;
    Boolean highlighted;
} Effector;

typedef struct effector_list {
    Effector *head;
    struct effector_list *tail;
} EffectorList;

typedef struct object_trigger_function_list {
    ObjectRole role;
    double (*trigger)(ContentionScheduler *cs, Object *o);
    struct object_trigger_function_list *next;
} ObjectTriggerFunctionList;

typedef struct schema {
    char *name;
    char *goal;
    ObjectTriggerFunctionList *objectTriggers;
    double (*effectorTrigger)(ContentionScheduler *cs, Effector *e);
    double activation[MaxCycles];
    Boolean selected;
    Boolean intended;
    Boolean executed;
    Boolean scope[MAX_ROLES-1];
    Boolean resource[MAX_RESOURCES];
    ActionId corresponding_action;
    GoalList *subgoals;
    struct schema_list *competitors;
    SchemaLevel level;
    Boolean highlighted;
} Schema;

typedef struct schema_list {
    Schema *head;
    struct schema_list *tail;
} SchemaList;

typedef struct actions {
    ActionId            action;
    Object             *obj1;
    Effector           *effector1;
    Effector           *effector2;
    struct actions     *next;
} Actions;

#define minAct 0.0
#define maxAct 1.0

typedef struct cs_parameters {
    double restAct;
    double selectionThreshold;
    double persistence;
    double goalCutoff;
    double goalPrecision;
    MathsType maths;
    Boolean interventions;
    double parental[MAX_COMPONENTS];
    double environmental[MAX_COMPONENTS];
    double self[MAX_COMPONENTS];
    double lateral[MAX_COMPONENTS];
    double noiseSD[MAX_COMPONENTS];
    double wm_excite;
    double wm_inhibit;
    double wm_noise;
    double wm_persistence;
    double wm_inhibition;
} CsParameters;

extern CsParameters parameter;
extern ContentionScheduler *scheduler;
extern FILE *messages[MAX_MESSAGE_TYPES];
extern char *roleName[MAX_ROLES];

/******************************************************************************/
/* Function declarations: ----------------------------------------------------*/

/* In cs_scheduler.c: --------------------------------------------------------*/

extern ContentionScheduler *cs_create();
extern Boolean cs_initialise(ContentionScheduler *cs);
extern void cs_reinitialise(ContentionScheduler *cs, char *logfile);
extern void cs_destroy(ContentionScheduler *cs);
extern void cs_run_task(ContentionScheduler *cs, void *task, int time_out);

/******************************************************************************/

extern double situationTriggersObject(Object *o, ObjectRole r);

/* In component.c: */

extern Object  *findObject(ContentionScheduler *cs, char *n);
extern Effector *findEffector(ContentionScheduler *cs, char *n);
extern Effector *bestEmptyHand(ContentionScheduler *cs);
extern Effector *bestNonemptyHand(ContentionScheduler *cs);

extern int random_index(int listLength);

/* In effector.c */

extern Effector *initialise_effector(char n[], Boolean s);
extern void destroy_effector(Effector *e);

/* In xframe.c */

extern int usertime();
extern int systime();
extern void update_cycle_display(ContentionScheduler *cs);

extern double initial_activation(ComponentType role);
extern void normalise_schema_activations(ContentionScheduler *cs);
extern void normalise_object_activations(ContentionScheduler *cs);
extern void normalise_effector_activations(ContentionScheduler *cs);

extern void schema_calculate_activation(ContentionScheduler *cs, Schema *this);
extern void object_calculate_activation(ContentionScheduler *cs, Object *this);
extern void effector_calculate_activation(ContentionScheduler *cs, Effector *this);

extern Boolean object_can_serve_role(Object *this, ObjectRole role);
extern Boolean schema_goal_achieved(ContentionScheduler *cs, Schema *this);

extern void message(MessageType c, ContentionScheduler *cs, char *b);

/******************************************************************************/
/* Schema installation functions: *********************************************/

extern void install_schema_subgoal(Schema *s, char *goal_name, GoalType postcondition(), GoalType precondition());
extern void install_schema_scope(Schema *s, ObjectRole role);
extern void install_schema_resource(Schema *s, Resource r);
extern Schema *install_schema(ContentionScheduler *cs, char *schema_name, char *goal_name, ActionId act, SchemaLevel level);
extern void install_schema_object_trigger(Schema *s, ObjectRole role, double object_trigger());
extern void install_schema_effector_trigger(Schema *s, double effector_trigger());

/* Object installation functions: *********************************************/

extern Object *objectCreate(ContentionScheduler *cs, char *object_name, unsigned int features, int state);
extern void objectDestroy(ContentionScheduler *cs, Object *o);

extern Object *objectInstall(ContentionScheduler *cs, char *object_name, unsigned int features, int state);
extern Object *objectInstallItem(ContentionScheduler *cs, char *name, unsigned int features);
extern void objectInstallAttachment(Object *object, Object *attached);
extern void objectInstallContents(Object *object, Object *content);
extern void objectInstallCover(Object *object, Object *cover);

extern void objectRemoveCover(Object *this, Object *cover);
extern void objectDeleteAttached(Object *base, Object *attached);
extern void objectDeleteCarried(Object *carrier, Object *carried);
extern void objectDeleteContained(Object *container, Object *contained);

extern Object *objectGetAttachment(Object *this);
extern Object *objectGetCover(Object *this);
extern Object *objectGetContent(Object *this);
extern Object *objectGetMostActive(ContentionScheduler *cs, ObjectRole r);

extern void objectSetPosition(Object *object, double x, double y);
extern void objectSetObjectTrigger(Object *object, double function());

extern ObjectList *objectListDeleteElement(ObjectList *ol, Object *theme);
extern ObjectList *objectListAppendElement(ObjectList *ol, Object *theme);

extern void objectBindToRole(ContentionScheduler *cs, Object *object, ObjectRole role);

extern Boolean isInSubordinateContactWith(Object *this, Object *theme);

extern double object_has_feature(Object *this, int feature);
extern Object *objectGetByName(ContentionScheduler *cs, char *name);

/* Effector installation functions: *******************************************/

extern Effector *effectorInstall(ContentionScheduler *cs, char *effector_name);

extern void effectorDeleteHeld(Effector *effector, Object *theme);
extern void effectorAddItemToContents(Effector *hand, Object *item);
extern void effectorDiscardContents(ContentionScheduler *cs, Effector *effector);
extern int effectorDiscardCoordinate(Effector *hand);

extern Object *effectorGetContents(Effector *hand);
extern Effector *effectorGetWorst(ContentionScheduler *cs);
extern Effector *effectorGetOtherHand(ContentionScheduler *cs, Effector *hand);

/******************************************************************************/

extern void initialise_schemas(ContentionScheduler *cs);
extern void initialise_objects(ContentionScheduler *cs);
extern void initialise_effectors(ContentionScheduler *cs);
extern void reinitialise_schemas(ContentionScheduler *cs);
extern void reinitialise_objects(ContentionScheduler *cs);
extern void reinitialise_effectors(ContentionScheduler *cs);
extern void random_seed_load(char *filename, ContentionScheduler *cs);
extern void random_seed_save(char *filename, ContentionScheduler *cs);

extern void cs_initialise_task(ContentionScheduler *cs, TaskId id);

extern char *task_names[TASK_MAX];

extern Schema *selected_parent(ContentionScheduler *cs, Schema *this);

/******************************************************************************/

extern GoalType isUsed(Object *o);
extern GoalType wasBounced(Object *o);
extern GoalType isHeld(Object *this);
extern GoalType isAttached(Object *this);
extern GoalType isCarried(Object *this);
extern GoalType isContained(Object *this);
extern GoalType isCovered(Object *this);
extern GoalType isEmptyObject(Object *this);
extern GoalType isFullObject(Object *this);
extern GoalType isClosed(Object *this);
extern GoalType isOpen(Object *this);
extern GoalType hasAttachment(Object *this);
extern GoalType isContainer(Object *o);
extern GoalType isTable(Object *o);
extern GoalType isTool(Object *o);
extern GoalType isSharp(Object *o);
extern GoalType isSphere(Object *o);
extern GoalType isEmptyContainer(Object *o);

/******************************************************************************/

extern Boolean isEmptyEffectorBoolean(Effector *this);
extern Boolean isFullEffectorBoolean(Effector *this);
extern Boolean isEmptyObjectBoolean(Object *this);
extern Boolean isFullObjectBoolean(Object *this);
extern Boolean isOpenBoolean(Object *this);
extern Boolean isClosedBoolean(Object *this);
extern Boolean isAttachedBoolean(Object *this);
extern Boolean isContainedBoolean(Object *this);
extern Boolean isHeldBoolean(Object *this);
extern Boolean isCarriedBoolean(Object *this);
extern Boolean isCoveredBoolean(Object *this);
extern Boolean isContainerBoolean(Object *this);
extern Boolean isFlatBoolean(Object *this);
extern Boolean isToolBoolean(Object *this);
extern Boolean isFluidBoolean(Object *this);
extern Boolean isObject(Object *this, char *objectName);
extern Boolean isOnTable(Object *this);

/******************************************************************************/

extern double isEmptyEffectorFuzzy(Effector *this);
extern double isFullEffectorFuzzy(Effector *this);

/******************************************************************************/

extern char *string_copy(char *buffer);

/******************************************************************************/

extern GoalType goalCurrentSourceFixated(ContentionScheduler *cs);
extern GoalType goalCurrentTargetFixated(ContentionScheduler *cs);
extern GoalType goalCurrentThemeFixated(ContentionScheduler *cs);
extern GoalType goalCurrentImplementFixated(ContentionScheduler *cs);

extern GoalType goalCurrentSourceHeld(ContentionScheduler *cs);
extern GoalType goalCurrentTargetHeld(ContentionScheduler *cs);
extern GoalType goalCurrentThemeHeld(ContentionScheduler *cs);
extern GoalType goalCurrentImplementHeld(ContentionScheduler *cs);

extern GoalType goalCurrentSourceOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentTargetOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentThemeOnTable(ContentionScheduler *cs);
extern GoalType goalCurrentImplementOnTable(ContentionScheduler *cs);

/******************************************************************************/

/* Error functions: */

extern void errorObjectNull(ContentionScheduler *cs, char *role, char *action);
extern void errorObjectIncorrect(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3);
extern void errorObjectState(ContentionScheduler *cs, char *template, char *arg1, char *arg2, char *arg3);

extern void errorEffectorNull(ContentionScheduler *cs, char *action);
extern void errorEffectorIncorrect(ContentionScheduler *cs, char *template, char *e1n, char *e2n);
extern void errorEffectorState(ContentionScheduler *cs, char *template, char *effector);

/* Action logging: */

extern void actionMessage(ContentionScheduler *cs, char *action, char *message);
extern void record_intervention(ContentionScheduler *cs, char *action);
extern void record_action(ContentionScheduler *cs, char *action, char *arg1, char *arg2, char *arg3);

/******************************************************************************/

extern void traceEffectorPreAction(ContentionScheduler *cs, Effector *this);
extern void traceEffectorPostAction(ContentionScheduler *cs, Effector *this);

extern void traceObjectPreAction(ContentionScheduler *cs, Object *this);
extern void traceObjectPostAction(ContentionScheduler *cs, Object *this);
extern void traceObjectPostCreate(ContentionScheduler *cs, Object *this);
extern void traceObjectPostNameChange(ContentionScheduler *cs, Object *this);

/******************************************************************************/

#include <glib.h>

/* Generic triggering and goal related functions: *****************************/

/******************************************************************************/

extern GoalType goalTrue(ContentionScheduler *cs);
extern GoalType goalFalse(ContentionScheduler *cs);
extern GoalType goalHandsEmpty(ContentionScheduler *cs);
extern GoalType goalHandsFull(ContentionScheduler *cs);
extern GoalType goalHold(ContentionScheduler *cs, double meetsRestriction(Object *o));
extern GoalType goalOnTable(ContentionScheduler *cs, double meetsRestriction(Object *o));
extern GoalType goalFixated(ContentionScheduler *cs, double meetsRestriction(Object *o));

/******************************************************************************/
#if FALSE
extern double schemaFixateSourceTriggersSource(ContentionScheduler *cs, Object *o);
extern double schemaFixateSourceTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaFixateTargetTriggersTarget(ContentionScheduler *cs, Object *o);
extern double schemaFixateTargetTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaFixateImplementTriggersImplement(ContentionScheduler *cs, Object *o);
extern double schemaFixateImplementTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaFixateThemeTriggersTheme(ContentionScheduler *cs, Object *o);
extern double schemaFixateThemeTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPickUpTriggersSource(ContentionScheduler *cs, Object *o);
extern double schemaPickUpTriggersTarget(ContentionScheduler *cs, Object *o);
extern double schemaPickUpTriggersTheme(ContentionScheduler *cs, Object *o);
extern double schemaPickUpTriggersImplement(ContentionScheduler *cs, Object *o);
extern double schemaPickUpTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPutDownTriggersSource(ContentionScheduler *cs, Object *o);
extern double schemaPutDownTriggersTarget(ContentionScheduler *cs, Object *o);
extern double schemaPutDownTriggersTheme(ContentionScheduler *cs, Object *o);
extern double schemaPutDownTriggersImplement(ContentionScheduler *cs, Object *o);
extern double schemaPutDownTriggersEffector(ContentionScheduler *cs, Effector *e);

/*----------------------------------------------------------------------------*/

extern double schemaPickUpSourceTriggersSource(ContentionScheduler *cs, Object *o);
extern double schemaPickUpSourceTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPickUpTargetTriggersTarget(ContentionScheduler *cs, Object *o);
extern double schemaPickUpTargetTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPickUpImplementTriggersImplement(ContentionScheduler *cs, Object *o);
extern double schemaPickUpImplementTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPickUpThemeTriggersTheme(ContentionScheduler *cs, Object *o);
extern double schemaPickUpThemeTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPutDownSourceTriggersSource(ContentionScheduler *cs, Object *o);
extern double schemaPutDownSourceTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPutDownTargetTriggersTarget(ContentionScheduler *cs, Object *o);
extern double schemaPutDownTargetTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPutDownImplementTriggersImplement(ContentionScheduler *cs, Object *o);
extern double schemaPutDownImplementTriggersEffector(ContentionScheduler *cs, Effector *e);

extern double schemaPutDownThemeTriggersTheme(ContentionScheduler *cs, Object *o);
extern double schemaPutDownThemeTriggersEffector(ContentionScheduler *cs, Effector *e);
#endif

extern void step_selections(ContentionScheduler *cs, void *task);

extern void cs_reset_intended_schema_subgoals(ContentionScheduler *cs);
extern void dump_activations(ContentionScheduler *cs);
extern Boolean all_intended_schemata_complete(ContentionScheduler *cs);

extern void cs_schema_set_intended_all(ContentionScheduler *cs, Boolean setting);
extern void cs_schema_set_intended(ContentionScheduler *cs, char *name, Boolean setting);
extern void cs_schema_set_intended_goal(ContentionScheduler *cs, char *name, Boolean setting);
extern Boolean cs_schema_is_selected(ContentionScheduler *cs);

/******************************************************************************/
#endif
