
#include "ss.h"

/******************************************************************************/
/* General goal test functions ************************************************/

GoalType goalTrue(ContentionScheduler *cs)
{
     return(GOAL_TRUE);
}

GoalType goalFalse(ContentionScheduler *cs)
{
     return(GOAL_FALSE);
}

/*----------------------------------------------------------------------------*/

GoalType goalHandsEmpty(ContentionScheduler *cs)
{
    EffectorList *el;
    GoalType r = GOAL_TRUE;

    for (el = cs->effectors; el != NULL; el = el->tail) {
        r = FZAND(r, isEmptyEffectorFuzzy(el->head));
    }
    return(r);
}

/*----------------------------------------------------------------------------*/

GoalType goalHandsFull(ContentionScheduler *cs)
{
    EffectorList *el;
    GoalType r = GOAL_FALSE;

    for (el = cs->effectors; el != NULL; el = el->tail) {
        r = FZOR(r, isFullEffectorFuzzy(el->head));
    }
    return(r);
}

/*----------------------------------------------------------------------------*/

GoalType goalHold(ContentionScheduler *cs, double meetsRestriction(Object *o))
{
    EffectorList *el;
    ObjectList *ol;
    GoalType r = GOAL_FALSE;

    for (el = cs->effectors; el != NULL; el = el->tail) {
        for (ol = el->head->itemsHeld; ol != NULL; ol = ol->tail) {
            r = FZOR(r, (GoalType) meetsRestriction(ol->head));
        }
    }
    return(r);
}

GoalType goalOnTable(ContentionScheduler *cs, double meetsRestriction(Object *o))
{
    ObjectList *os;
    GoalType found = GOAL_FALSE;

    for (os = cs->objects; os != NULL; os = os->tail) {
        if (isOnTable(os->head)) {
            found = FZOR(found, (GoalType) meetsRestriction(os->head));
        }
    }
    return(found);
}

GoalType goalFixated(ContentionScheduler *cs, double meetsRestriction(Object *o))
{
    return((cs->fixated_object == NULL) ? GOAL_FALSE : (GoalType) meetsRestriction(cs->fixated_object));
}

/******************************************************************************/

static GoalType goalCurrentRoleFixated(ContentionScheduler *cs, ObjectRole role)
{
    if (cs->fixated_object == NULL) {
        return(GOAL_FALSE);
    }
    else if (cs->fixated_object == cs->current_object[role]) {
        return(GOAL_TRUE);
    }
    else {
        return(GOAL_FALSE);
    }
}

GoalType goalCurrentSourceFixated(ContentionScheduler *cs)
{
    return(goalCurrentRoleFixated(cs, SOURCE));
}

GoalType goalCurrentTargetFixated(ContentionScheduler *cs)
{
    return(goalCurrentRoleFixated(cs, TARGET));
}

GoalType goalCurrentThemeFixated(ContentionScheduler *cs)
{
    return(goalCurrentRoleFixated(cs, THEME));
}

GoalType goalCurrentImplementFixated(ContentionScheduler *cs)
{
    return(goalCurrentRoleFixated(cs, IMPLEMENT));
}

/*----------------------------------------------------------------------------*/

static GoalType goalCurrentRoleHeld(ContentionScheduler *cs, ObjectRole role)
{
    if (cs->current_object[role] == NULL) {
        return(GOAL_FALSE);
    }
    else {
        return(isHeld(cs->current_object[role]));
    }
}

GoalType goalCurrentSourceHeld(ContentionScheduler *cs)
{
    return(goalCurrentRoleHeld(cs, SOURCE));
}

GoalType goalCurrentTargetHeld(ContentionScheduler *cs)
{
    return(goalCurrentRoleHeld(cs, TARGET));
}

GoalType goalCurrentThemeHeld(ContentionScheduler *cs)
{
    return(goalCurrentRoleHeld(cs, THEME));
}

GoalType goalCurrentImplementHeld(ContentionScheduler *cs)
{
    return(goalCurrentRoleHeld(cs, IMPLEMENT));
}

/*----------------------------------------------------------------------------*/

static GoalType goalCurrentRoleOnTable(ContentionScheduler *cs, ObjectRole role)
{
    if (cs->current_object[role] == NULL) {
        return(GOAL_FALSE);
    }
    else {
        return(FZNOT(isHeld(cs->current_object[role])));
    }
}

GoalType goalCurrentSourceOnTable(ContentionScheduler *cs)
{
    return(goalCurrentRoleOnTable(cs, SOURCE));
}

GoalType goalCurrentTargetOnTable(ContentionScheduler *cs)
{
    return(goalCurrentRoleOnTable(cs, TARGET));
}

/******************************************************************************/

/* As of Wed Jul 30 13:57:09 2003 this generic function replaces all of the   */
/* situationTriggersX functions. The generic function ensures symmetry        */
/* between objects and schemas by constructing the degree of schema           */
/* triggering by the environment from the schemaXTriggersObject functions.    */
/* It is marginally less efficient, because we scan all arguments roles and   */
/* then ignore those that don't contribute, but this loss in efficiency is    */
/* worth the gain in consistency. There is one case where the generic         */
/* function is less than perfectly adequate: the case of the swap schema,     */
/* but this is never more than a distractor schema, so it can be safely       */
/* ignored. One solution would be to replace swap with separate swap_source   */
/* swap_target swap_implement and swap_theme schemas.                         */

/* The function returns the average relative salience (i.e., scaled to        */
/* between 0 and 1) if there is at least one salient object, or -1.0 if       */
/* there are no salient objects.                                              */

double situationTriggerFromObjectTrigger(ContentionScheduler *cs, Schema *this)
{
    ObjectTriggerFunctionList *tf;
    ObjectList *os;
    double tmp, cred, sum = 0.0;
    int s = 0;

    for (tf = this->objectTriggers; tf != NULL; tf = tf->next) {
        tmp = minAct - parameter.restAct;
        for (os = cs->objects; os; os = os->tail) {
            cred = tf->trigger(cs, os->head);
            tmp = MAX(tmp, cred * (os->head->activation[tf->role][cs->cycle-1]-parameter.restAct));
	}
        sum += tmp; s++;
    }

    if (s > 0) {
        return(sum / (double) (s * (maxAct - minAct)));
    }
    else {
        return(-1.0);
    }
}

#if FALSE

/******************************************************************************/
/* General object triggering functions ---------------------------------------*/
/******************************************************************************/

double schemaFixateSourceTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(((object != cs->fixated_object) && object_can_serve_role(object, SOURCE)) ? 1.0 : 0.0);
}

/*----------------------------------------------------------------------------*/

double schemaFixateTargetTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(((object != cs->fixated_object) && object_can_serve_role(object, TARGET)) ? 1.0 : 0.0);
}    

/*----------------------------------------------------------------------------*/

double schemaFixateImplementTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return((object != cs->fixated_object) ? FZAND(isTool(object), FZNOT(isSharp(object))) : 0.0);
}    

/*----------------------------------------------------------------------------*/

double schemaFixateThemeTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(((object != cs->fixated_object) && object_can_serve_role(object, THEME)) ? 1.0 : 0.0);
}

/*----------------------------------------------------------------------------*/

double schemaPickUpTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(FZAND(FZNOT(isHeld(object)), FZAND(FZNOT(isAttached(object)), FZNOT(isContained(object)))));
}

/*----------------------------------------------------------------------------*/

double schemaPickUpTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(FZAND(FZNOT(isHeld(object)), FZAND(FZNOT(isAttached(object)), FZNOT(isContained(object)))));
}    

/*----------------------------------------------------------------------------*/

double schemaPickUpTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(FZAND(FZNOT(isHeld(object)), FZAND(FZNOT(isAttached(object)), FZNOT(isContained(object)))));
}    

/*----------------------------------------------------------------------------*/

double schemaPickUpTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(FZAND(FZNOT(isHeld(object)), FZAND(FZNOT(isAttached(object)), FZNOT(isContained(object)))));
}

/******************************************************************************/

double schemaPutDownTriggersSource(ContentionScheduler *cs, Object *object)
{
    return(isHeld(object));
}

/*----------------------------------------------------------------------------*/

double schemaPutDownTriggersTarget(ContentionScheduler *cs, Object *object)
{
    return(isHeld(object));
}    

/*----------------------------------------------------------------------------*/

double schemaPutDownTriggersImplement(ContentionScheduler *cs, Object *object)
{
    return(isHeld(object));
}    

/*----------------------------------------------------------------------------*/

double schemaPutDownTriggersTheme(ContentionScheduler *cs, Object *object)
{
    return(isHeld(object));
}

/******************************************************************************/
/* General effector triggering functions -------------------------------------*/
/******************************************************************************/

double schemaFixateSourceTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(parameter.restAct);
}

/*----------------------------------------------------------------------------*/

double schemaFixateTargetTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(parameter.restAct);
}

/*----------------------------------------------------------------------------*/

double schemaFixateImplementTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(parameter.restAct);
}

/*----------------------------------------------------------------------------*/

double schemaFixateThemeTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(parameter.restAct);
}

/*----------------------------------------------------------------------------*/

double schemaPickUpTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(cs->fixated_object == NULL ? 0.0 : isEmptyEffectorFuzzy(effector));
}

/*----------------------------------------------------------------------------*/

double schemaPutDownTriggersEffector(ContentionScheduler *cs, Effector *effector)
{
    return(isFullEffectorFuzzy(effector));
}

/******************************************************************************/

double schemaPickUpSourceTriggersSource(ContentionScheduler *cs, Object *o)
{
    return(parameter.restAct);
}

double schemaPickUpSourceTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(isEmptyEffectorFuzzy(e));
}

/*----------------------------------------------------------------------------*/

double schemaPickUpTargetTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return(parameter.restAct);
}

double schemaPickUpTargetTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(isEmptyEffectorFuzzy(e));
}

/*----------------------------------------------------------------------------*/

double schemaPickUpImplementTriggersImplement(ContentionScheduler *cs, Object *o)
{
    return(parameter.restAct);
}

double schemaPickUpImplementTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(isEmptyEffectorFuzzy(e));
}

/*----------------------------------------------------------------------------*/

double schemaPickUpThemeTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return(1.0);
}

double schemaPickUpThemeTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(isEmptyEffectorFuzzy(e));
}

/******************************************************************************/

static Boolean object_in_role_is_held(ContentionScheduler *cs, Effector *e, ObjectRole role)
{
    return((cs->current_object[role] == NULL) ? FALSE : (cs->current_object[role]->holder == e));
}

double schemaPutDownSourceTriggersSource(ContentionScheduler *cs, Object *o)
{
    return((cs->current_object[SOURCE] == o) ? isHeld(o) : 0.0);
}

double schemaPutDownTargetTriggersTarget(ContentionScheduler *cs, Object *o)
{
    return((cs->current_object[TARGET] == o) ? isHeld(o) : 0.0);
}

double schemaPutDownThemeTriggersTheme(ContentionScheduler *cs, Object *o)
{
    return((cs->current_object[THEME] == o) ? isHeld(o) : 0.0);
}

double schemaPutDownImplementTriggersImplement(ContentionScheduler *cs, Object *o)
{
    return((cs->current_object[IMPLEMENT] == o) ? isHeld(o) : 0.0);
}

double schemaPutDownSourceTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(object_in_role_is_held(cs, e, SOURCE) ? 1.0 : 0.0);
}

double schemaPutDownTargetTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(object_in_role_is_held(cs, e, TARGET) ? 1.0 : 0.0);
}

double schemaPutDownThemeTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(object_in_role_is_held(cs, e, THEME) ? 1.0 : 0.0);
}

double schemaPutDownImplementTriggersEffector(ContentionScheduler *cs, Effector *e)
{
    return(object_in_role_is_held(cs, e, IMPLEMENT) ? 1.0 : 0.0);
}

/******************************************************************************/
#endif
