
 #include "ss.h"

extern void install_null_task_schemas(ContentionScheduler *cs);
extern void install_null_task_objects(ContentionScheduler *cs);
extern void install_null_task_effectors(ContentionScheduler *cs);
extern void write_null_task_object_attributes(FILE *fp, Object *this);
extern void write_null_task_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_null_task();

extern void install_wcst_schemas(ContentionScheduler *cs);
extern void install_wcst_basic_objects(ContentionScheduler *cs);
extern void install_wcst_effectors(ContentionScheduler *cs);
extern void write_wcst_object_attributes(FILE *fp, Object *this);
extern void write_wcst_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_wcst_task();

extern void install_tol_schemas(ContentionScheduler *cs);
extern void install_tol_basic_objects(ContentionScheduler *cs);
extern void install_tol_effectors(ContentionScheduler *cs);
extern void write_tol_object_attributes(FILE *fp, Object *this);
extern void write_tol_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_tol_task();

extern void install_rng_schemas(ContentionScheduler *cs);
extern void install_rng_basic_objects(ContentionScheduler *cs);
extern void install_rng_effectors(ContentionScheduler *cs);
extern void write_rng_object_attributes(FILE *fp, Object *this);
extern void write_rng_effector_attributes(FILE *fp, Effector *effector);
extern void analyse_rng_task();

char *task_names[TASK_MAX] = {
    "None",
    "WCST",
    "ToL",
    "RNG",
};

void cs_initialise_task(ContentionScheduler *cs, TaskId id)
{
    cs->cycle = 0;
    cs->task.id = id;
    switch (cs->task.id) {
        case TASK_NONE: {
            cs->task.install_schemas = install_null_task_schemas;
            cs->task.install_objects = install_null_task_objects;
            cs->task.install_effectors = install_null_task_effectors;
            cs->task.write_object_attributes = write_null_task_object_attributes;
            cs->task.write_effector_attributes = write_null_task_effector_attributes;
            cs->task.analyse = analyse_null_task;
fprintf(stdout, "Loaded task NONE\n");
            break;
        }
        case TASK_WCST: {
            cs->task.install_schemas = install_wcst_schemas;
            cs->task.install_objects = install_wcst_basic_objects;
            cs->task.install_effectors = install_wcst_effectors;
            cs->task.write_object_attributes = write_wcst_object_attributes;
            cs->task.write_effector_attributes = write_wcst_effector_attributes;
            cs->task.analyse = analyse_wcst_task;
fprintf(stdout, "Loaded task WCST\n");
            break;
        }
        case TASK_TOL: {
            cs->task.install_schemas = install_tol_schemas;
            cs->task.install_objects = install_tol_basic_objects;
            cs->task.install_effectors = install_tol_effectors;
            cs->task.write_object_attributes = write_tol_object_attributes;
            cs->task.write_effector_attributes = write_tol_effector_attributes;
            cs->task.analyse = analyse_tol_task;
fprintf(stdout, "Loaded task ToL\n");
            break;
        }
        case TASK_RNG: {
            cs->task.install_schemas = install_rng_schemas;
            cs->task.install_objects = install_rng_basic_objects;
            cs->task.install_effectors = install_rng_effectors;
            cs->task.write_object_attributes = write_rng_object_attributes;
            cs->task.write_effector_attributes = write_rng_effector_attributes;
            cs->task.analyse = analyse_rng_task;
fprintf(stdout, "Loaded task RNG\n");
            break;
        }
        default: {
            fprintf(stdout, "ERROR: Task definition not found\n");
        }
    }
}
