#define RNG_TRIALS_MAX  100
#define RNG_BLOCKS        5

typedef struct rng_disks {
    double            x, y;
    int               id;
    int               colour;
    struct rng_disks *next;
} RngDisks;

typedef struct rng_task_data {
    RngDisks  *disks;
    int        event_count;
    int        trial_count;
    int        block_count;
    Boolean    trial_complete;
    RngDisks  *target_selected;
    int        sequence[RNG_BLOCKS][RNG_TRIALS_MAX];
    double     score_r[RNG_BLOCKS];
    double     score_rng[RNG_BLOCKS];
    double     score_rr[RNG_BLOCKS];
    double     score_oa[RNG_BLOCKS];
    double     score_aa[RNG_BLOCKS];
    double     score_tpi[RNG_BLOCKS];
    double     score_rg[RNG_BLOCKS];
} RngTaskData;

extern RngTaskData *rng_create_task();
extern void rng_destroy_task(RngTaskData *task);
extern void rng_initialise_task(RngTaskData *task);
extern void rng_initialise_next_trial(ContentionScheduler *cs, RngTaskData *task);
extern Boolean rng_trial_complete(RngTaskData *task);
extern Boolean rng_task_complete(RngTaskData *task);
extern void rng_step_event(ContentionScheduler *cs, RngTaskData *task);
extern void rng_synchronise_world(ContentionScheduler *cs, RngTaskData *task);
extern void rng_dump_analyses(FILE *fp, RngTaskData *task);
extern Boolean rng_disk_is_selected(ContentionScheduler *cs);
extern void rng_initialise_wm(ContentionScheduler *cs);

extern Object *rng_select_best_target(ContentionScheduler *cs);
extern Object *rng_get_matching_disk(ContentionScheduler *cs, char *label);
extern Boolean rng_disk_selected(ContentionScheduler *cs);
extern Boolean rng_disk_attended(ContentionScheduler *cs);
extern RngDisks *rng_get_disk(RngTaskData *task, char *l);
extern void rng_select_disk(ContentionScheduler *cs, RngTaskData *task, char *l);
extern char *rng_get_disk_label(Object *disk);
