/*******************************************************************************

    File:     rng_effector.c
    Author:   Rick Cooper
    Created:  27/06/08
    Edited:   27/06/08
    Contents: Create/maintain effectors for the RNG task

    Public procedures:
        void install_rng_effectors(ContentionScheduler *cs)
        void write_rng_effector_attributes(FILE *fp, Effector *effector)

*******************************************************************************/

#include "ss.h"
#include "rng.h"

/******************************************************************************/
/* Section 1: Install effectors ----------------------------------------------*/

void install_rng_effectors(ContentionScheduler *cs)
{
    effectorInstall(cs, "leftHand");
    effectorInstall(cs, "rightHand");
}

/******************************************************************************/
/* Section 2: Write effector attributes (for debugging) ----------------------*/

void write_rng_effector_attributes(FILE *fp, Effector *effector)
{
    ObjectList *os;

    if (effector->itemsHeld) {
	fprintf(fp, "holds");
	for (os = effector->itemsHeld; os; os = os->tail) {
	    if (os->tail) {
		fprintf(fp, " %s,", os->head->name);
	    }
	    else {
		fprintf(fp, " %s", os->head->name);
	    }
	}
    }
    else {
	fprintf(fp, "empty");
    }
}

/******************************************************************************/
