/*******************************************************************************

    File:     ss.c
    Author:   Rick Cooper
    Created:  06/04/07
    Edited:   06/04/07
    Contents: A text interface to the SS model

    Public procedures:
        int main(int argc, char **argv)

*******************************************************************************/

#include "ss.h"
#include <stdlib.h>
#include <time.h>
#define TASK TASK_RNG

/******************************************************************************/

void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning)
{
//    fprintf(stdout, "%4d: WARNING (%s): %s\n", cs->cycle, type, warning);
}

void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on)
{
//    fprintf(stdout, "%4d: %s %s\n", cs->cycle, schema_name, (on ? "selected" : "deselected"));
}

void action_log_add_action(ContentionScheduler *cs, char *act, char *message)
{
//    fprintf(stdout, "%4d: ACTION - %s: %s\n", cs->cycle, act, message);
}

/******************************************************************************/
/* Section 2: Main -----------------------------------------------------------*/

int main(int argc, char **argv)
{
    long t0 = (long) time(NULL);
    GlobalVariables *globals;

    srand((int) t0);

    if ((globals = world_create(TASK)) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating world\n", argv[0]);
    }
    else if (!task_create(globals)) {
        fprintf(stderr, "%s: Aborting - Task creation failed\n", argv[0]);
    }
    else if ((globals->cs = cs_create()) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating contention scheduler\n", argv[0]);
    }
    else if (!world_initialise_task(globals)) {
        fprintf(stderr, "%s: Aborting - World initialisation failed\n", argv[0]);
    }
    else if (!cs_initialise(globals->cs)) {
        fprintf(stderr, "%s: Aborting - Contention scheduler initialisation failed\n", argv[0]);
    }
    else {
        task_run_block(globals);
        task_dump_results(stdout, globals);
    }

    cs_destroy(globals->cs);
    task_destroy(globals);
    world_destroy(globals);

    exit(0);
}

/******************************************************************************/
