#include <stdio.h>
#include <stdlib.h>

typedef enum boolean {FALSE, TRUE} Boolean;

typedef enum task_id {TASK_NONE, TASK_WCST, TASK_TOL, TASK_RNG, TASK_MAX} TaskId;

typedef enum action_id {ACTION_NONE, ACTION_PICK_UP_THEME, ACTION_PUT_DOWN_TARGET,
    ACTION_ATTEND_TARGET, ACTION_DISENGAGE_TARGET, ACTION_SELECT_TARGET
} ActionId;

#include "cs.h"
#include "chunk.h"

typedef struct global_variables {
    void           *window;
    void           *world_da;
    void           *analysis_da;
    void           *cs_graphs[3];
    void           *wm_graph;
    void           *gc;
    Boolean         dynamic_canvases;
    TaskId          task;
    void           *task_data;
    ContentionScheduler *cs;
} GlobalVariables;

/******************************************************************************/
/* Defined in world.c: -------------------------------------------------------*/

extern Boolean task_create(GlobalVariables *globals);
extern void task_destroy(GlobalVariables *globals);
extern GlobalVariables *world_create();
extern void world_destroy(GlobalVariables *globals);
extern Boolean world_initialise_task(GlobalVariables *globals);
extern void world_initialise_next_trial(GlobalVariables *globals);
extern Boolean world_trial_complete(GlobalVariables *globals);
extern Boolean world_task_complete(GlobalVariables *globals);
extern void world_step_event(GlobalVariables *globals);

/******************************************************************************/

extern void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning);
extern void action_log_add_action(ContentionScheduler *cs, char *act, char *message);
extern void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on);

/******************************************************************************/
/* Defined in task.c: --------------------------------------------------------*/

extern void task_run_block(GlobalVariables *globals);
extern void task_dump_results(FILE *fp, GlobalVariables *globals);

/******************************************************************************/

extern void temporary_schema_reject_current(ContentionScheduler *cs);
extern char *temporary_schema_get(ContentionScheduler *cs);
extern void temporary_schema_set(ContentionScheduler *cs, char *s);

/******************************************************************************/
/* In wm_*.c: ----------------------------------------------------------------*/

extern void wm_destroy(ContentionScheduler *cs);
extern void wm_add(ContentionScheduler *cs, Chunk *chunk);
extern void wm_delete(ContentionScheduler *cs, Chunk *template);
extern void wm_delete_all(ContentionScheduler *cs, Chunk *template);
extern Chunk *wm_query(ContentionScheduler *cs, Chunk *template);
extern Boolean wm_match(ContentionScheduler *cs, Chunk *template);
extern void wm_step_activations(ContentionScheduler *cs);

/******************************************************************************/
