/*******************************************************************************

    File:     ss_client.c
    Author:   Rick Cooper
    Created:  06/04/07
    Edited:   06/04/07
    Contents: An express client for the SS model

    Public procedures:
        int main(int argc, char **argv)

*******************************************************************************/

#include "ss.h"
#include <stdlib.h>
#include <time.h>
#define TASK TASK_WCST
#include "wcst.h"
#include <libexpress.h>

// Note: This file is WCST-specific. It shouldn't be!

/******************************************************************************/

void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning)
{
}

void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on)
{
}

void action_log_add_action(ContentionScheduler *cs, char *act, char *message)
{
}

/******************************************************************************/
/* Section 2: Main -----------------------------------------------------------*/

int main(int argc, char **argv)
{
    long t0 = (long) time(NULL);
    GlobalVariables *globals;
    int uid, hit;

    srand((int) t0);

    if ((globals = world_create(TASK)) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating world\n", argv[0]);
    }
    else if (!task_create(globals)) {
        fprintf(stderr, "%s: Aborting - Task creation failed\n", argv[0]);
    }
    else if ((globals->cs = cs_create()) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating contention scheduler\n", argv[0]);
    }
    else if (!world_initialise_task(globals)) {
        fprintf(stderr, "%s: Aborting - World initialisation failed\n", argv[0]);
    }
    else if (!cs_initialise(globals->cs)) {
        fprintf(stderr, "%s: Aborting - Contention scheduler initialisation failed\n", argv[0]);
    }
    else {
        WcstTaskData *task = globals->task_data;

        server_set(SP_HOST, "dropbear.psyc.bbk.ac.uk");
        server_set(SP_PAGE, "/expts/ss_wcst_64A_luce_n01_p08/express.cgi");
        server_set(SP_VERBOSE, stderr);

        server_init(argc, argv);

        server_declare_variable("trial", VAR_WIV, VAR_INT, &hit);
        server_declare_variable("wm_noise", VAR_BIV, VAR_DOUBLE, &parameter.wm_noise);
        server_declare_variable("wm_persistence", VAR_BIV, VAR_DOUBLE, &parameter.wm_persistence);
        server_declare_variable("wm_inhibit", VAR_BIV, VAR_DOUBLE, &parameter.wm_inhibit);
        server_declare_variable("wm_excite", VAR_BIV, VAR_DOUBLE, &parameter.wm_excite);

        server_declare_variable("num_correct", VAR_DV, VAR_INT, &task->score_total_correct_sorts);
        server_declare_variable("num_categories", VAR_DV, VAR_INT, &task->score_categories_obtained);
        server_declare_variable("num_errors_classical_perseverations", VAR_DV, VAR_INT, &task->score_errors_perseverations);
        server_declare_variable("num_errors_ppr", VAR_DV, VAR_INT, &task->score_errors_ppr);
        server_declare_variable("num_errors_ppc", VAR_DV, VAR_INT, &task->score_errors_ppc);
        server_declare_variable("num_errors_set_loss", VAR_DV, VAR_INT, &task->score_errors_set_loss);
        server_declare_variable("num_errors_other", VAR_DV, VAR_INT, &task->score_errors_other);
        
        while ((uid = server_next_participant()) > 0) {
            world_initialise_task(globals);
            cs_reinitialise(globals->cs, NULL);
            do {
                task_run_block(globals);
            } while (server_next_hit(uid) > 0);
        }

        server_free();
    }

    cs_destroy(globals->cs);
    task_destroy(globals);
    world_destroy(globals);

    exit(0);
}

/******************************************************************************/
