/*******************************************************************************

    File:     task.c
    Author:   Rick Cooper
    Created:  06/04/07
    Edited:   06/04/07
    Contents: 

    Public procedures:
        void task_create(GlobalVariables *globals)
        void task_destroy(GlobalVariables *globals)

*******************************************************************************/

#include "ss.h"
#include "wcst.h"
#include "tol.h"
#include "rng.h"

/******************************************************************************/
/* Section 1: Data structure creation/destruction ----------------------------*/

Boolean task_create(GlobalVariables *globals)
{
    switch (globals->task) {
        case TASK_WCST: {
            globals->task_data = wcst_create_task();
            break;
        }
        case TASK_TOL: {
            globals->task_data = tol_create_task();
            break;
        }
        case TASK_RNG: {
            globals->task_data = rng_create_task();
            break;
        }
        default: {
             globals->task_data = NULL;
        }
    }
    return(TRUE);
}

/*----------------------------------------------------------------------------*/

void task_destroy(GlobalVariables *globals)
{
    switch (globals->task) {
        case TASK_WCST: {
            wcst_destroy_task((WcstTaskData *)globals->task_data);
	    break;
        }
        case TASK_TOL: {
            tol_destroy_task((TolTaskData *)globals->task_data);
	    break;
        }
        case TASK_RNG: {
            rng_destroy_task((RngTaskData *)globals->task_data);
	    break;
        }
        default: {
        }
    }
}

/******************************************************************************/
/* Section 2: -*/

void task_run_block(GlobalVariables *globals)
{
    if (globals->task == TASK_NONE) {
        fprintf(stdout, "No task is selected.\nPlease select a task before continuing.");
    }
    else if (world_task_complete(globals)) {
        fprintf(stdout, "The task is complete.");
    }
    else {
        while (!world_task_complete(globals)) {
            while (!world_trial_complete(globals)) {
                world_step_event(globals);
            }
            if (!world_task_complete(globals)) {
                world_initialise_next_trial(globals);
            }
        }
    }
}

void task_dump_results(FILE *fp, GlobalVariables *globals)
{
    switch (globals->task) {
        case TASK_WCST: {
            wcst_dump_analyses(fp, globals->task_data);
            break;
        }
        case TASK_TOL: {
            tol_dump_analyses(fp, globals->task_data);
            break;
        }
        case TASK_RNG: {
            rng_dump_analyses(fp, globals->task_data);
            break;
        }
        default: {
            break;
        }
    }
}

/******************************************************************************/
