#define TOL_PROBLEM_MAX 8

typedef struct ball_position {
    int          peg;
    int          position;
} BallPosition;

typedef struct tol_configuration {
    BallPosition red_ball;
    BallPosition green_ball;
    BallPosition blue_ball;
} TolConfiguration;

typedef struct tol_problem {
    TolConfiguration initial_state;
    TolConfiguration current_state;
    TolConfiguration desired_state;
    int moves_minimum;
    int moves_actual;
    int rule_breaks;
    int balls_correct;
    Boolean trial_done;
} TolProblem;

typedef struct tol_task_data {
    int        event_count;
    int        trial_count;
    TolProblem problems[TOL_PROBLEM_MAX];
} TolTaskData;

extern TolTaskData *tol_create_task();
extern void tol_destroy_task(TolTaskData *task);
extern void tol_initialise_task(TolTaskData *task);
extern void tol_initialise_next_trial(ContentionScheduler *cs, TolTaskData *task);
extern Boolean tol_trial_complete(TolTaskData *task);
extern Boolean tol_task_complete(TolTaskData *task);
extern void tol_step_event(ContentionScheduler *cs, TolTaskData *task);
extern void tol_synchronise_world(ContentionScheduler *cs, TolTaskData *task);
extern void tol_action_pick_up(TolTaskData *task, Object *theme, Effector *effector);
extern void tol_action_put_down(TolTaskData *task, Object *theme, Object *target);
extern void tol_dump_analyses(FILE *fp, TolTaskData *task);
extern void tol_initialise_wm(ContentionScheduler *cs);
