/*******************************************************************************

    File:     tol_object.c
    Author:   Rick Cooper
    Created:  30/03/07
    Edited:   07/04/07
    Contents: Create/maintain ToL object representations

    Public procedures:
        void install_tol_basic_objects(ContentionScheduler *cs)
        void write_tol_object_attributes(FILE *fp, Object *this)

*******************************************************************************/

#include "ss.h"
#include "tol.h"

/******************************************************************************/
/* Section 1: Install ToL object representations -----------------------------*/

void install_tol_basic_objects(ContentionScheduler *cs)
{
    Object *this;

    /* Three current-state pegs: */
    this = objectInstall(cs, "peg_current_1", IS_PEG | IS_APPARATUS | IS_NUMBER_ONE, 0);
    objectSetPosition(this, 3.0, 0.0);
    this = objectInstall(cs, "peg_current_2", IS_PEG | IS_APPARATUS | IS_NUMBER_TWO, 0);
    objectSetPosition(this, 2.0, 0.0);
    this = objectInstall(cs, "peg_current_3", IS_PEG | IS_APPARATUS | IS_NUMBER_THREE, 0);
    objectSetPosition(this, 1.0, 0.0);

    /* Three balls (positions are set when a trial is initiated): */
    this = objectInstall(cs, "ball_current_red", IS_BALL | IS_APPARATUS | IS_COLOUR_RED, 0);
    this = objectInstall(cs, "ball_current_green", IS_BALL | IS_APPARATUS | IS_COLOUR_GREEN, 0);
    this = objectInstall(cs, "ball_current_blue", IS_BALL | IS_APPARATUS | IS_COLOUR_BLUE, 0);

    /* Three goal-state pegs: */
    this = objectInstall(cs, "peg_goal_1", IS_PEG | IS_GOAL_STATE | IS_NUMBER_ONE, 0);
    objectSetPosition(this, 3.0, 0.0);
    this = objectInstall(cs, "peg_goal_2", IS_PEG | IS_GOAL_STATE | IS_NUMBER_TWO, 0);
    objectSetPosition(this, 2.0, 0.0);
    this = objectInstall(cs, "peg_goal_3", IS_PEG | IS_GOAL_STATE| IS_NUMBER_THREE, 0);
    objectSetPosition(this, 1.0, 0.0);

    /* Three balls (positions are set when a trial is initiated): */
    this = objectInstall(cs, "ball_goal_red", IS_BALL | IS_GOAL_STATE | IS_COLOUR_RED, 0);
    this = objectInstall(cs, "ball_goal_green", IS_BALL | IS_GOAL_STATE | IS_COLOUR_GREEN, 0);
    this = objectInstall(cs, "ball_goal_blue", IS_BALL | IS_GOAL_STATE | IS_COLOUR_BLUE, 0);
}

/******************************************************************************/
/* Section 2: Write the object attributes (for debugging) --------------------*/

void write_tol_object_attributes(FILE *fp, Object *this)
{
    if (object_has_feature(this, IS_APPARATUS)) {
        fprintf(fp, "apparatus: ");
    }
    else if (object_has_feature(this, IS_GOAL_STATE)) {
        fprintf(fp, "goal: ");
    }

    else if (object_has_feature(this, IS_NUMBER_TWO)) {
        fprintf(fp, "two ");
    }
    else if (object_has_feature(this, IS_NUMBER_THREE)) {
        fprintf(fp, "three ");
    }

    if (object_has_feature(this, IS_COLOUR_RED)) {
        fprintf(fp, "red ");
    }
    else if (object_has_feature(this, IS_COLOUR_GREEN)) {
        fprintf(fp, "green ");
    }
    else if (object_has_feature(this, IS_COLOUR_BLUE)) {
        fprintf(fp, "blue ");
    }

    if (object_has_feature(this, IS_PEG)) {
        fprintf(fp, "peg ");
    }
    else if (object_has_feature(this, IS_BALL)) {
        fprintf(fp, "ball ");
    }
}

/******************************************************************************/
