/*******************************************************************************

    File:     wcst_object.c
    Author:   Rick Cooper
    Created:  30/03/07
    Edited:   30/03/07
    Contents: Create/maintain WCST object representations

    Public procedures:
        void install_wcst_basic_objects(ContentionScheduler *cs)
        void write_wcst_object_attributes(FILE *fp, Object *this)

*******************************************************************************/

#include "ss.h"
#include "wcst.h"

/******************************************************************************/
/* Section 1: Install WCST object representations ----------------------------*/

void install_wcst_basic_objects(ContentionScheduler *cs)
{
    Object *this;

    /* Four target cards: */
    this = objectInstall(cs, "target1", IS_FORM_TRIANGLE | IS_COLOUR_RED | IS_NUMBER_ONE, 0);
    objectSetPosition(this, 1.0, 1.0);
    this = objectInstall(cs, "target2", IS_FORM_STAR | IS_COLOUR_GREEN | IS_NUMBER_TWO, 0);
    objectSetPosition(this, 1.0, 2.0);
    this = objectInstall(cs, "target3", IS_FORM_CROSS | IS_COLOUR_YELLOW | IS_NUMBER_THREE, 0);
    objectSetPosition(this, 1.0, 3.0);
    this = objectInstall(cs, "target4", IS_FORM_CIRCLE | IS_COLOUR_BLUE | IS_NUMBER_FOUR, 0);
    objectSetPosition(this, 1.0, 4.0);

    /* Four places for sorted cards (initially with no features): */
    this = objectInstall(cs, "sorted1", 0, 0);
    objectSetPosition(this, 2.0, 1.0);
    this = objectInstall(cs, "sorted2", 0, 0);
    objectSetPosition(this, 2.0, 2.0);
    this = objectInstall(cs, "sorted3", 0, 0);
    objectSetPosition(this, 2.0, 3.0);
    this = objectInstall(cs, "sorted4", 0, 0);
    objectSetPosition(this, 2.0, 4.0);

    /* The to-be-sorted card: */
    this = objectInstall(cs, "current", 0, 0);
    objectSetPosition(this, 3.0, 1.0);

    /* The feedback, when it appears: */
    this = objectInstall(cs, "feedback", 0, 0);
    objectSetPosition(this, 3.0, 4.0);
}

/******************************************************************************/
/* Section 2: Write the object attributes (for debugging) --------------------*/

void write_wcst_object_attributes(FILE *fp, Object *this)
{
    if (object_has_feature(this, IS_NUMBER_ONE)) {
        fprintf(fp, "one ");
    }
    else if (object_has_feature(this, IS_NUMBER_TWO)) {
        fprintf(fp, "two ");
    }
    else if (object_has_feature(this, IS_NUMBER_THREE)) {
        fprintf(fp, "three ");
    }
    else if (object_has_feature(this, IS_NUMBER_FOUR)) {
        fprintf(fp, "four ");
    }

    if (object_has_feature(this, IS_COLOUR_RED)) {
        fprintf(fp, "red ");
    }
    else if (object_has_feature(this, IS_COLOUR_GREEN)) {
        fprintf(fp, "green ");
    }
    else if (object_has_feature(this, IS_COLOUR_YELLOW)) {
        fprintf(fp, "yellow ");
    }
    else if (object_has_feature(this, IS_COLOUR_BLUE)) {
        fprintf(fp, "blue ");
    }

    if (object_has_feature(this, IS_FORM_TRIANGLE)) {
        fprintf(fp, "triangle ");
    }
    else if (object_has_feature(this, IS_FORM_STAR)) {
        fprintf(fp, "star ");
    }
    else if (object_has_feature(this, IS_FORM_CROSS)) {
        fprintf(fp, "cross ");
    }
    else if (object_has_feature(this, IS_FORM_CIRCLE)) {
        fprintf(fp, "circle ");
    }
}

/******************************************************************************/
