
#include "ss.h"
#include "math.h"

#ifdef WM_CONTINUOUS
#undef LUCE_CHOICE

static int wm_next_colour()
{
    static colour = 1;

    if (colour < 26) {
        colour++;
    }
    else {
        colour = 2;
    }
    return(colour);
}

Wme *wm_install_node(ContentionScheduler *cs, char *name)
{
    WmeList *new;

    if (cs->wmes != NULL) {
        /* The node is already the first WME; Update its timestamp and return: */
        if (strcmp(cs->wmes->head->schema_name, name) == 0) {
            cs->wmes->head->timestamp = cs->cycle;
            return(cs->wmes->head);
        }
        else {
            WmeList *prev = cs->wmes;
            WmeList *tmp = prev->tail;

            /* If the node is already present, move it to the front of the list, */
            /* update its timestamp and return: */

            while ((tmp != NULL) && (strcmp(tmp->head->schema_name, name) != 0)) {
                prev = tmp;
                tmp = tmp->tail;
            }
            if (tmp != NULL) {
                tmp->head->timestamp = cs->cycle;
                prev->tail = tmp->tail;
                tmp->tail = cs->wmes;
                cs->wmes = tmp;
                return(tmp->head);
            }
        }
    }

    /* Otherwise create a new element: */
    if ((new = (WmeList *)malloc(sizeof(WmeList))) == NULL) {
	fprintf(stderr, "MEMORY ALLOCATION FAILURE: wm_install_node.1\n");
    }
    else if ((new->head = (Wme *)malloc(sizeof(Wme))) == NULL) {
	fprintf(stderr, "MEMORY ALLOCATION FAILURE: wm_install_node.2\n");
    }
    else {
        new->head->activation[cs->cycle] = parameter.restAct;
        new->head->schema_name = string_copy(name);
        new->head->input = 0.0;
        new->head->colour = wm_next_colour();
        new->head->timestamp = cs->cycle;
        new->tail = cs->wmes;
        cs->wmes = new;
fprintf(stdout, "Activation of %s initialised to %6.3f on %d\n", new->head->schema_name, new->head->activation[cs->cycle], cs->cycle);
    }
    return(new->head);
}

static void laterally_inhibit_competitors(ContentionScheduler *cs, double p)
{
    // p is the extent of lateral inhibition

    WmeList *wl1, *wl2;

    for (wl1 = cs->wmes; wl1 != NULL; wl1 = wl1->tail) {
        for (wl2 = cs->wmes; wl2 != NULL; wl2 = wl2->tail) {
            if (wl1 != wl2) {
                wl1->head->input -= (p * (wl2->head->activation[cs->cycle]-parameter.restAct));
	    }
	}
    }
}

void wm_recalculate_activations(ContentionScheduler *cs)
{
    extern double wm_new_activation(double previous, double input);

    WmeList *wl;

    laterally_inhibit_competitors(cs, parameter.wm_inhibition);

    for (wl = cs->wmes; wl != NULL; wl = wl->tail) {
        wl->head->activation[cs->cycle] = wm_new_activation(wl->head->activation[cs->cycle], wl->head->input);
	wl->head->input = 0.0;
// if (cs->cycle < 10) {
//fprintf(stdout, "%d: %s recalculated: %6.4f\n", cs->cycle, wl->head->schema_name, wl->head->activation[cs->cycle]);
// }
    }
}

void wm_step_activations(ContentionScheduler *cs)
{
    WmeList *wl;

    for (wl = cs->wmes; wl != NULL; wl = wl->tail) {
        wl->head->activation[cs->cycle] = wl->head->activation[cs->cycle-1];
// if (cs->cycle < 10) {
//fprintf(stdout, "Activation of %s updated to %6.3f on %d\n", wl->head->schema_name, wl->head->activation[cs->cycle], cs->cycle);
// }
    }
}

void wm_excite_node(ContentionScheduler *cs, Wme *w, double e)
{
    w->input += e;
}

#ifdef LUCE_CHOICE
Wme *wm_select_most_active_element(ContentionScheduler *cs)
{
    WmeList *wl;
    double sum = 0.0, r;

    sum = 0;
    for (wl = cs->wmes; wl != NULL; wl = wl->tail) {
        sum = sum + wl->head->activation[cs->cycle];
    }
    r = sum * random() / pow(2, 31);

    sum = 0;
    for (wl = cs->wmes; wl != NULL; wl = wl->tail) {
        sum = sum + wl->head->activation[cs->cycle];
        if (sum > r) {
            return(wl->head);
        }
    }
    return(NULL);
}
#else
Wme *wm_select_most_active_element(ContentionScheduler *cs)
{
    WmeList *wl;
    Wme *w = NULL;
    double m = -0.001;

    for (wl = cs->wmes; wl != NULL; wl = wl->tail) {
        if (wl->head->activation[cs->cycle] > m) {
            w = wl->head;
            m = w->activation[cs->cycle];
        }
    }
    return(w);
}
#endif

void wm_destroy(ContentionScheduler *cs)
{
    while (cs->wmes != NULL) {
        WmeList *this = cs->wmes;
        free(this->head);
        free(this);
        cs->wmes = this->tail;
    }
}

#endif
