/*******************************************************************************

    File:     x_analysis.c
    Author:   Rick Cooper
    Created:  29/03/07
    Edited:   29/03/07
    Contents: Routines for analysing and displaying behaviour of the SS model in X

    Public procedures:
        GtkWidget *x_analysis_create_page(GlobalVariables *globals)
        void x_analysis_redraw(GlobalVariables *globals)

*******************************************************************************/

#include "ss.h"
#include <gtk/gtk.h>
#include "lib_gtkx.h"

#include "wcst.h"
#include "tol.h"
#include "rng.h"

extern void x_wcst_redraw_analysis(GlobalVariables *globals, WcstTaskData *task);
extern void x_tol_redraw_analysis(GlobalVariables *globals, TolTaskData *task);
extern void x_rng_redraw_analysis(GlobalVariables *globals, RngTaskData *task);

/******************************************************************************/
/* Section 1: Redraw the world canvas ----------------------------------------*/

static void x_clear_analysis_canvas(GlobalVariables *globals, int width, int height)
{
    GdkPixmap *canvas = (GdkPixmap *)g_object_get_data(G_OBJECT(globals->world_da), "pixmap");

    gdk_gc_set_foreground(globals->gc, &colour[26]);
    gdk_draw_rectangle(canvas, globals->gc, TRUE, 0, 0, width, height);
    gdk_gc_set_foreground(globals->gc, &colour[0]);
}

/*----------------------------------------------------------------------------*/

void x_analysis_redraw(GlobalVariables *globals)
{
    GtkWidget *da = globals->analysis_da;
    GdkPixmap *canvas;
    int width, height;

    gtk_widget_realize(da);
    canvas = (GdkPixmap *)g_object_get_data(G_OBJECT(da), "pixmap");
    gdk_drawable_get_size((GdkDrawable *)canvas, &width, &height);

    switch (globals->task) {
        case TASK_WCST: {
            x_wcst_redraw_analysis(globals, (WcstTaskData *)globals->task_data);
            break;
        }
        case TASK_TOL: {
            x_tol_redraw_analysis(globals, (TolTaskData *)globals->task_data);
            break;
        }
        case TASK_RNG: {
            x_rng_redraw_analysis(globals, (RngTaskData *)globals->task_data);
            break;
        }
        default: {
            x_clear_analysis_canvas(globals, width, height);
            break;
        }
    }
    gdk_draw_drawable(da->window, globals->gc, canvas, 0, 0, 0, 0, width, height);
    gtk_widget_queue_draw_area(da, 0, 0, width, height);
    gtkx_flush_events();
}

/******************************************************************************/
/* Section 2: Create the canvas and set canvas event handlers ----------------*/

static int analysis_canvas_event_configure(GtkWidget *da, GdkEventConfigure *event, GlobalVariables *globals)
{
    gtkx_canvas_reallocate(da);
    x_analysis_redraw(globals);
    return(1);
}

/*----------------------------------------------------------------------------*/

GtkWidget *x_analysis_create_page(GlobalVariables *globals)
{
    GtkWidget *da;
    
    da = gtk_drawing_area_new();
    g_signal_connect(G_OBJECT(da), "configure_event", G_CALLBACK(analysis_canvas_event_configure), globals);
    g_signal_connect(G_OBJECT(da), "expose_event", G_CALLBACK(gtkx_canvas_event_expose), NULL);
    g_signal_connect(G_OBJECT(da), "destroy", G_CALLBACK(gtkx_canvas_callback_destroy), NULL);

    globals->analysis_da = da;

    return(da);
}

/*****************************************************************************/
