/*******************************************************************************

    File:     x_cs_log.c
    Author:   Rick Cooper
    Created:  03/04/07
    Edited:   03/04/07
    Contents: 

    Public procedures:

*******************************************************************************/

#include "ss.h"
#include <math.h>
#include <gtk/gtk.h>
#include "lib_gtkx.h"

/******************************************************************************/

void action_log_add_warning(ContentionScheduler *cs, char *type, char *warning)
{
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(cs->cl_actions));
    GtkTreeIter iter;

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                    0, cs->cycle,
                    1, type,
                    3, warning,
                    -1);
}

void action_log_add_selection(ContentionScheduler *cs, char *schema_name, Boolean on)
{
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(cs->cl_actions));
    GtkTreeIter iter;
    char buffer[32];

    g_snprintf(buffer, 32, "%c%s", (on ? '+' : '-'), schema_name);
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                    0, cs->cycle,
                    1, buffer,
                    -1);
}

void action_log_add_action(ContentionScheduler *cs, char *act, char *message)
{
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(cs->cl_actions));
    GtkTreeIter iter;

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                    0, cs->cycle,
                    2, act,
                    3, message,
                    -1);
}

/******************************************************************************/

GtkWidget *x_cs_logs_create_page(ContentionScheduler *cs)
{
    GtkWidget *page;
    GtkListStore *list;
    GtkTreeViewColumn *column;
    GtkCellRenderer *renderer;

    page = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    list = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

    cs->cl_actions = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Cycle", renderer, "text", 0, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(cs->cl_actions), column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Selections", renderer, "text", 1, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(cs->cl_actions), column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Actions", renderer, "text", 2, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(cs->cl_actions), column);

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Messages", renderer, "text", 3, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(cs->cl_actions), column);

    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(cs->cl_actions), TRUE);
    gtk_container_add(GTK_CONTAINER(page), cs->cl_actions);
    gtk_widget_show(cs->cl_actions);

    return(page);
}

/******************************************************************************/
