/*******************************************************************************

    File:     x_task.c
    Author:   Rick Cooper
    Created:  29/03/07
    Edited:   29/03/07
    Contents: Routines for running SS models in X

    Public procedures:
        void x_task_initialise(GtkWidget *caller, GlobalVariables *globals)
        void x_task_run_event(GtkWidget *caller, GlobalVariables *globals)
        void x_task_run_trial(GtkWidget *caller, GlobalVariables *globals)
        void x_task_run_block(GtkWidget *caller, GlobalVariables *globals)

*******************************************************************************/

#include "ss.h"
#include <gtk/gtk.h>
#include "lib_gtkx.h"

extern void x_redraw_canvases(GlobalVariables *globals, Boolean force);

/******************************************************************************/
/* Section 1: Handle model initialise/step/run events ------------------------*/

void x_task_initialise(GtkWidget *caller, GlobalVariables *globals)
{
    gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(globals->cs->cl_actions)));
    if (globals->task == TASK_NONE) {
        io_message(globals->window, "No task is selected.\nPlease select a task before continuing.");
    }
    else {
        world_initialise_task(globals);
        cs_reinitialise(globals->cs, "tmp.log");
        x_redraw_canvases(globals, TRUE);
    }
}

/*----------------------------------------------------------------------------*/

void x_task_run_event(GtkWidget *caller, GlobalVariables *globals)
{
    if (globals->task == TASK_NONE) {
        io_message(globals->window, "No task is selected.\nPlease select a task before continuing.");
    }
    else if (!world_trial_complete(globals)) {
        world_step_event(globals);
        x_redraw_canvases(globals, TRUE);
    }
    else if (world_task_complete(globals)) {
        io_message(globals->window, "The task is complete.");
    }
    else if (io_query(globals->window, "The current trial is complete.\nInitialise the next trial?")) {
        world_initialise_next_trial(globals);
        x_redraw_canvases(globals, TRUE);
    }
}

/*----------------------------------------------------------------------------*/

void x_task_run_trial(GtkWidget *caller, GlobalVariables *globals)
{
    if (globals->task == TASK_NONE) {
        io_message(globals->window, "No task is selected.\nPlease select a task before continuing.");
    }
    else if (!world_trial_complete(globals)) {
        while (!world_trial_complete(globals)) {
            world_step_event(globals);
            x_redraw_canvases(globals, globals->dynamic_canvases);
	}
        x_redraw_canvases(globals, !globals->dynamic_canvases);
    }
    else if (world_task_complete(globals)) {
        io_message(globals->window, "The task is complete.");
    }
    else if (io_query(globals->window, "The current trial is complete.\nRun the next trial?")) {
        world_initialise_next_trial(globals);
        x_redraw_canvases(globals, globals->dynamic_canvases);
        while (!world_trial_complete(globals)) {
            world_step_event(globals);
            x_redraw_canvases(globals, globals->dynamic_canvases);
	}
        x_redraw_canvases(globals, !globals->dynamic_canvases);
    }
}

/*----------------------------------------------------------------------------*/

void x_task_run_block(GtkWidget *caller, GlobalVariables *globals)
{
    if (globals->task == TASK_NONE) {
        io_message(globals->window, "No task is selected.\nPlease select a task before continuing.");
    }
    else if (world_task_complete(globals)) {
        io_message(globals->window, "The task is complete.");
    }
    else {
        while (!world_task_complete(globals)) {
            while (!world_trial_complete(globals)) {
                world_step_event(globals);
                x_redraw_canvases(globals, globals->dynamic_canvases);
            }
            if (!world_task_complete(globals)) {
                world_initialise_next_trial(globals);
                x_redraw_canvases(globals, globals->dynamic_canvases);
            }
        }
        x_redraw_canvases(globals, !globals->dynamic_canvases);
    }
}

/******************************************************************************/
