/*******************************************************************************

    File:     x_wcst.c
    Author:   Rick Cooper
    Created:  28/03/07
    Edited:   29/03/07
    Contents: Routines displaying the WCST in X

    Public procedures:
        void x_wcst_redraw_world(GlobalVariables *globals, WcstTaskData *task)

*******************************************************************************/

#include "ss.h"
#include <string.h>
#include <gtk/gtk.h>
#include "lib_gtkx.h"
#include "wcst.h"
#include "math.h"

/******************************************************************************/
/* Section 1: Redraw the world -----------------------------------------------*/

static void draw_shape(GdkPixmap *canvas, GdkGC *gc, int x, int y, int form)
{
    if (form == 1) { /* Triangle */
        GdkPoint points[3];

        points[0].x = x-12;
        points[0].y = y+10;

        points[1].x = x+12;
        points[1].y = y+10;

        points[2].x = x;
        points[2].y = y-10;

        gdk_draw_polygon(canvas, gc, TRUE, points, 3);
    }
    else if (form == 2) {
        GdkPoint points[10];
        double sin54 = sin(M_PI * 54.0 / 180.0);
        double sin72 = sin(M_PI * 72.0 / 180.0);
        double cos54 = cos(M_PI * 54.0 / 180.0);
        double cos72 = cos(M_PI * 72.0 / 180.0);
        double tan18 = tan(M_PI * 18.0 / 180.0);
        double tan54 = tan(M_PI * 54.0 / 180.0);
        int d = 24;
        int y0 = y - d / 2.0;
        int y1 = y0 + (sin72 - cos54) * d;
        int y2 = y0 + cos54 * d;
        int y3 = y0 + (sin72 - cos72/tan54) * d;
        int y4 = y0 + sin72 * d;
        int x0 = x - d / 2.0;
        int x1 = x0 + (1.0 - sin54) * d;
        int x2 = x0 + (sin54 - 0.5) * d;
        int x3 = x0 + (0.5 - tan18 * (sin72-cos54)) * d;
        int x4 = x0 + 0.5 * d;
        int x5 = x0 + (0.5 + tan18 * (sin72-cos54)) * d;
        int x6 = x0 + (1.5 - sin54) * d;
        int x7 = x0 + sin54 * d;
        int x8 = x0 + d;

        points[0].x = x4;
        points[0].y = y0;

        points[1].x = x5;
        points[1].y = y1;

        points[2].x = x8;
        points[2].y = y1;

        points[3].x = x6;
        points[3].y = y2;

        points[4].x = x7;
        points[4].y = y4;

        points[5].x = x4;
        points[5].y = y3;

        points[6].x = x1;
        points[6].y = y4;

        points[7].x = x2;
        points[7].y = y2;

        points[8].x = x0;
        points[8].y = y1;

        points[9].x = x3;
        points[9].y = y1;

        gdk_draw_polygon(canvas, gc, TRUE, points, 10);
    }
    else if (form == 3) {
        gdk_draw_rectangle(canvas, gc, TRUE, x-3, y-11, 7, 23);
        gdk_draw_rectangle(canvas, gc, TRUE, x-11, y-3, 23, 7);
    }
    else if (form == 4) {
        gdk_draw_arc(canvas, gc, TRUE, x-10, y-10, 21, 21, 0, 23040);
    }
}

/*----------------------------------------------------------------------------*/

static void draw_card(GdkPixmap *canvas, GdkGC *gc,int x, int y, Card *card)
{
    /* Get the colour: */
    if (card->colour == 1) {        /* Red */
        gdk_gc_set_foreground(gc, &colour[18]);
    }
    else if (card->colour == 2) {   /* Green */
        gdk_gc_set_foreground(gc, &colour[6]);
    }
    else if (card->colour == 3) {   /* Yellow */
        gdk_gc_set_foreground(gc, &colour[24]);
    }
    else if (card->colour == 4) {   /* Blue */
        gdk_gc_set_foreground(gc, &colour[2]);
    }
    else {                          /* Grey */
        gdk_gc_set_foreground(gc, &colour[13]);
    }

    if (card->number == 1) {
        draw_shape(canvas, gc, x+50, y+50, card->form);
    }
    else if (card->number == 2) {
        draw_shape(canvas, gc, x+25, y+25, card->form);
        draw_shape(canvas, gc, x+75, y+75, card->form);
    }
    else if (card->number == 3) {
        draw_shape(canvas, gc, x+25, y+25, card->form);
        draw_shape(canvas, gc, x+75, y+25, card->form);
        draw_shape(canvas, gc, x+50, y+75, card->form);
    }
    else if (card->number == 4) {
        draw_shape(canvas, gc, x+25, y+25, card->form);
        draw_shape(canvas, gc, x+25, y+75, card->form);
        draw_shape(canvas, gc, x+75, y+25, card->form);
        draw_shape(canvas, gc, x+75, y+75, card->form);
    }
    else {                              /* Draw a checked pattern: */
        int x1, y1;
        for (x1 = x; x1 < x + 100; x1 = x1 + 20) {
            for (y1 = y; y1 < y + 100; y1 = y1 + 20) {
                gdk_draw_rectangle(canvas, gc, TRUE, x1, y1, 10, 10);
                gdk_draw_rectangle(canvas, gc, TRUE, x1+10, y1+10, 10, 10);
            }
        }
    }

    /* Now draw the card border: */
    gdk_gc_set_foreground(gc, &colour[0]);
    gdk_draw_rectangle(canvas, gc, FALSE, x, y, 100, 100);
}

/*----------------------------------------------------------------------------*/

void x_wcst_redraw_world(GlobalVariables *globals, WcstTaskData *task)
{
    GdkPixmap *canvas = (GdkPixmap *)g_object_get_data(G_OBJECT(globals->world_da), "pixmap");
    PangoLayout *layout = gtk_widget_create_pango_layout(globals->world_da, "");
    int width, height, i;
    char buffer[64];

    gdk_drawable_get_size((GdkDrawable *)canvas, &width, &height);

    gdk_gc_set_foreground(globals->gc, &colour[26]);
    gdk_draw_rectangle(canvas, globals->gc, TRUE, 0, 0, width, height);
    gdk_gc_set_foreground(globals->gc, &colour[0]);

    pango_layout_set_text(layout, "Target Cards:", strlen("Target Cards:"));
    pango_layout_set_width(layout, 50*PANGO_SCALE);
    gdk_draw_layout(canvas, globals->gc, 20, 80, layout);
    for (i = 0; i < 4; i++) {
        draw_card(canvas, globals->gc, 70 + (128 * i), 50, &(task->card_target[i]));
    }

    pango_layout_set_text(layout, "Sorted Cards:", strlen("Sorted Cards:"));
    pango_layout_set_width(layout, 50*PANGO_SCALE);
    gdk_draw_layout(canvas, globals->gc, 20, 230, layout);
    for (i = 0; i < 4; i++) {
        draw_card(canvas, globals->gc, 70 + (128 * i), 200, &(task->card_sorted[i]));
    }

    pango_layout_set_width(layout, 250*PANGO_SCALE);
    if (task->chosen_deck == 0) {
        draw_card(canvas, globals->gc, 262, 380, &(task->current_card));
    }
    if ((task->feedback != NULL) && (task->chosen_deck > 0)) {
        int x = task->chosen_deck * 128 - 8, w, h;
        pango_layout_set_text(layout, task->feedback, strlen(task->feedback));
        pango_layout_get_pixel_size(layout, &w, &h);
        gdk_draw_layout(canvas, globals->gc, x - w/2, 320, layout);
    }
    g_snprintf(buffer, 64, "Trial is %d.%02d", task->trial_count + 1, task->event_count + 1);
    pango_layout_set_text(layout, buffer, strlen(buffer));
    gdk_draw_layout(canvas, globals->gc, 20, 600, layout);

    g_object_unref(layout);
}

/******************************************************************************/
/* Section 2: Redraw analyses ------------------------------------------------*/

static void x_wcst_centre_text(GdkPixmap *canvas, GdkGC *gc, PangoLayout *layout, int x, int y, char *label)
{
    int w, h;

    pango_layout_set_text(layout, label, strlen(label));
    pango_layout_get_pixel_size(layout, &w, &h);
    gdk_draw_layout(canvas, gc, x-w/2, y, layout);
}

static void draw_score_sheet(GdkPixmap *canvas, GdkGC *gc, PangoLayout *layout, WcstTaskData *task, int x0, int y0)
{
    int i, x, y, c, seq = 0;

    gdk_gc_set_foreground(gc, &colour[0]);

    x_wcst_centre_text(canvas, gc, layout, x0+195, y0-15, "Trials:");
    c = task->score_categories_obtained;
    x = x0 + 230; y = y0 - 15;
    while (c >= 0) {
        x_wcst_centre_text(canvas, gc, layout, x, y, "C");
        x_wcst_centre_text(canvas, gc, layout, x+15, y, "F");
        x_wcst_centre_text(canvas, gc, layout, x+30, y, "N");
        c = c - 3; x = x + 45;
    }
    x = x0 + 230;
    for (c = 0; c < task->score_categories_obtained; c++) {
        gdk_draw_line(canvas, gc, x-4, y+10, x+4, y);
        x = x + 15;
    }

    gdk_draw_line(canvas, gc, x0, y0, x0+480, y0);
    gdk_draw_line(canvas, gc, x0, y0+320, x0+480, y0+320);
    for (i = 0; i < 9; i++) {
        gdk_draw_line(canvas, gc, x0+i*60, y0, x0+i*60, y0+320);
    }

    x = x0, y = y0-17;
    for (i = 0; i < NUM_CARDS; i++) {
        if (y > y0+300) {
            y = y0 + 3;
            x = x + 60;
        }
        else {
            y = y + 20;
        }
            
        if (i > task->trial_count) {
            /* Not yet sorted: draw in grey */
            gdk_gc_set_foreground(gc, &colour[13]);
        }
        else if ((i == task->trial_count) && !wcst_trial_complete(task)) {
            /* Not yet sorted: draw in grey */
            gdk_gc_set_foreground(gc, &colour[13]);
        }
        else if (task->correct[i]) {
            /* Correct: draw in black */
            gdk_gc_set_foreground(gc, &colour[0]);
            seq++;
            if (seq > 9) {
                gdk_draw_line(canvas, gc, x+4, y+15, x+56, y+15);
                seq = 0;
            }
        }
        else {
            /* An error: draw in red */
            gdk_gc_set_foreground(gc, &colour[18]);
            seq = 0;
        }

        x_wcst_centre_text(canvas, gc, layout, x+12, y+2, "C");
        x_wcst_centre_text(canvas, gc, layout, x+24, y+2, "F");
        x_wcst_centre_text(canvas, gc, layout, x+36, y+2, "N");
        x_wcst_centre_text(canvas, gc, layout, x+48, y+2, "O");

        if ((i < task->trial_count) || ((i == task->trial_count) && wcst_trial_complete(task))) {
            if (task->features_matched[i] & 1) {
                gdk_draw_line(canvas, gc, x+ 8, y+12, x+16, y+2);
            }
            if (task->features_matched[i] & 2) {
                gdk_draw_line(canvas, gc, x+20, y+12, x+28, y+2);
            }
            if (task->features_matched[i] & 4) {
                gdk_draw_line(canvas, gc, x+32, y+12, x+40, y+2);
            }
            if (task->features_matched[i] == 0) {
                gdk_draw_line(canvas, gc, x+44, y+12, x+52, y+2);
            }
        }

        if (task->perseverative_error[i]) {
            /* A perseverative error: Add a "**" to indicate perseveration */
            x_wcst_centre_text(canvas, gc, layout, x+6, y+4, "*");
            x_wcst_centre_text(canvas, gc, layout, x+55, y+4, "*");
        }
    }
}

/*----------------------------------------------------------------------------*/

static void draw_data_row(GdkPixmap *canvas, GdkGC *gc, PangoLayout *layout, int x0, int y0, char *label, int score)
{
    char buffer[64];
    int w, h;

    g_snprintf(buffer, 64, "%s:", label);
    pango_layout_set_text(layout, buffer, strlen(buffer));
    gdk_draw_layout(canvas, gc, x0+10, y0+4, layout);

    g_snprintf(buffer, 64, "%d", score);
    pango_layout_set_text(layout, buffer, strlen(buffer));
    pango_layout_get_pixel_size(layout, &w, &h);
    gdk_draw_layout(canvas, gc, x0+200 - w, y0+4, layout);
}

static void draw_summary_table(GdkPixmap *canvas, GdkGC *gc, PangoLayout *layout, WcstTaskData *task, int x0, int y0)
{
    int row_height = 20;
    int y1, r = 0, cards_sorted;

    cards_sorted = wcst_trial_complete(task) ? task->trial_count+1 :  task->trial_count;

    gdk_gc_set_foreground(gc, &colour[0]);
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Cards Sorted", cards_sorted); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Correct Sorts", task->score_total_correct_sorts); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Categories Obtained", task->score_categories_obtained); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Errors (Perseverations)", task->score_errors_perseverations); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Errors (PPC)", task->score_errors_ppc); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Errors (PPR)", task->score_errors_ppr); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Errors (Set Loss)", task->score_errors_set_loss); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));
    draw_data_row(canvas, gc, layout, x0, y0+r*row_height, "Errors (Other)", task->score_errors_other); r++;
    gdk_draw_line(canvas, gc, x0, y0+(r*row_height), x0+220, y0+(r*row_height));

    y1 = y0+(r*row_height);
    gdk_draw_line(canvas, gc, x0, y0, x0, y1);
    gdk_draw_line(canvas, gc, x0+170, y0, x0+170, y1);
    gdk_draw_line(canvas, gc, x0+220, y0, x0+220, y1);
}

/*----------------------------------------------------------------------------*/

void x_wcst_redraw_analysis(GlobalVariables *globals, WcstTaskData *task)
{
    GdkPixmap *canvas = (GdkPixmap *)g_object_get_data(G_OBJECT(globals->analysis_da), "pixmap");
    PangoLayout *layout = gtk_widget_create_pango_layout(globals->analysis_da, "");
    int width, height;

    gdk_drawable_get_size((GdkDrawable *)canvas, &width, &height);

    gdk_gc_set_foreground(globals->gc, &colour[26]);
    gdk_draw_rectangle(canvas, globals->gc, TRUE, 0, 0, width, height);
    gdk_gc_set_foreground(globals->gc, &colour[0]);

    draw_score_sheet(canvas, globals->gc, layout, task, (width - 480) / 2, 50);

    draw_summary_table(canvas, globals->gc, layout, task, (width - 480) / 2, 420);

    if (task->trial_count == 0) {
        gdk_gc_set_foreground(globals->gc, &colour[13]);
    }
    else {
        gdk_gc_set_foreground(globals->gc, &colour[0]);
    }
    pango_layout_set_text(layout, "Errors in red; Perseverative errors indicated by *...*", strlen("Errors in red; Perseverative errors indicated by *...*"));
    gdk_draw_layout(canvas, globals->gc, (width - 480) / 2, 380, layout);

    g_object_unref(layout);
}

/******************************************************************************/
