/*******************************************************************************

    File:     x_world.c
    Author:   Rick Cooper
    Created:  28/03/07
    Edited:   29/03/07
    Contents: Routines for displaying and running the SS world model in X

    Public procedures:
        GtkWidget *x_world_create_page(GlobalVariables *globals)
        void x_world_redraw(GlobalVariables *globals)

*******************************************************************************/

#include "ss.h"
#include <gtk/gtk.h>
#include "lib_gtkx.h"

#include "wcst.h"
#include "tol.h"
#include "rng.h"

extern void x_wcst_redraw_world(GlobalVariables *globals, cairo_t *cr, WcstTaskData *task);
extern void x_tol_redraw_world(GlobalVariables *globals, cairo_t *cr, TolTaskData *task);
extern void x_rng_redraw_world(GlobalVariables *globals, cairo_t *cr, RngTaskData *task);

/******************************************************************************/
/* Section 1: Redraw the world canvas ----------------------------------------*/

static void x_clear_world_canvas(GlobalVariables *globals, cairo_t *cr)
{
    GtkWidget *da = globals->world_da;
    int width = da->allocation.width;
    int height = da->allocation.height;

    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
    cairo_rectangle(cr, 0, 0, width, height);
    cairo_fill(cr);
}

/*----------------------------------------------------------------------------*/

void x_world_redraw(GlobalVariables *globals)
{
    GtkWidget *da = globals->world_da;
    cairo_t *cr = gdk_cairo_create(da->window);

    switch (globals->task) {
        case TASK_WCST: {
            x_wcst_redraw_world(globals, cr, (WcstTaskData *)globals->task_data);
            break;
        }
        case TASK_TOL: {
            x_tol_redraw_world(globals, cr, (TolTaskData *)globals->task_data);
            break;
        }
        case TASK_RNG: {
            x_rng_redraw_world(globals, cr, (RngTaskData *)globals->task_data);
            break;
        }
        default: {
            x_clear_world_canvas(globals, cr);
            break;
        }
    }
    cairo_destroy(cr);
}

/******************************************************************************/
/* Section 2: Create the canvas and set canvas event handlers ----------------*/

static int world_canvas_event_configure(GtkWidget *da, GdkEventConfigure *event, GlobalVariables *globals)
{
    x_world_redraw(globals);
    return(1);
}

/*----------------------------------------------------------------------------*/

GtkWidget *x_world_create_page(GlobalVariables *globals)
{
    GtkWidget *da;
    
    da = gtk_drawing_area_new();
    g_signal_connect(G_OBJECT(da), "expose_event", G_CALLBACK(world_canvas_event_configure), globals);
    g_signal_connect(G_OBJECT(da), "destroy", G_CALLBACK(gtkx_canvas_callback_destroy), NULL);

    globals->world_da = da;

    return(da);
}

/*****************************************************************************/
