/*******************************************************************************

    File:     xss.c
    Author:   Rick Cooper
    Created:  28/03/07
    Edited:   29/03/07
    Contents: An X interface to the SS model

    Public procedures:
        int main(int argc, char **argv)

*******************************************************************************/

#include "ss.h"
#include <stdlib.h>
#include <time.h>
#include <gtk/gtk.h>

#define TASK TASK_NONE

extern Boolean widgets_create(GlobalVariables *globals);
extern void widgets_initialise(GlobalVariables *globals);

/******************************************************************************/
/* Section 1: Main -----------------------------------------------------------*/

int main(int argc, char **argv)
{
    long t0 = (long) time(NULL);
    GlobalVariables *globals;

    gtk_set_locale();
    gtk_init(&argc, &argv);

    srand((int) t0);

    if ((globals = world_create(TASK)) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating world\n", argv[0]);
    }
    else if (!task_create(globals)) {
        fprintf(stderr, "%s: Aborting - Task creation failed\n", argv[0]);
    }
    else if ((globals->cs = cs_create()) == NULL) {
        fprintf(stderr, "%s: Aborting - Memory allocation failed while creating contention scheduler\n", argv[0]);
    }
    else if (!world_initialise_task(globals)) {
        fprintf(stderr, "%s: Aborting - World initialisation failed\n", argv[0]);
    }
    else if (!cs_initialise(globals->cs)) {
        fprintf(stderr, "%s: Aborting - Contention scheduler initialisation failed\n", argv[0]);
    }
    else if (widgets_create(globals)) {
        widgets_initialise(globals);
        gtk_widget_show(GTK_WIDGET(globals->window));
        gtk_main();
    }
    else {
        fprintf(stderr, "%s: Aborting - Widget creation failed\n", argv[0]);
    }

    cs_destroy(globals->cs);
    task_destroy(globals);
    world_destroy(globals);

    exit(0);
}

/******************************************************************************/


